This Google SCC Elastic Integration will help the users to check the findings associated with each asset obtained with the help of the Security Command Center, the information about finding sources, affected asset-related details and the Audit Logs for the Organization. These details will be fetched so that SOC analysts can see all the required data in one place. They can also visualize the data in the form of Dashboards.

#### **App Version:**
3.0.0

#### **Build Version:**
03

#### **Elastic Stack Versions:**
```
ElasticSearch 7.15.0 and Above
Kibana 7.15.0 and above
```
**( NOTE:  If a client has an existing ELK stack then we have to consider the version installed, 
But we will only support the version currently supported by Elastic.)**

We have tested the plugin in Linux (Centos7) with below versions
```
ElasticSearch 7.16.2
Logstash 7.16.2
Kibana 7.16.2
Filebeat 8.1.0 (custom filebeat)
Docker 20.10.6
Docker-compose 1.29.1
Go 1.18
```

Library/SDK versions
```
cloud.google.com/go/asset v1.0.2
cloud.google.com/go/logging v1.4.2
cloud.google.com/go/securitycenter v1.2.0
github.com/elastic/go-elasticsearch/v8 v8.4.0
github.com/joho/godotenv v1.4.0
github.com/robfig/cron v1.2.0
github.com/sirupsen/logrus v1.8.1
golang.org/x/oauth2 v0.0.0-20220622183110-fd043fe589d2
google.golang.org/api v0.93.0
google.golang.org/genproto v0.0.0-20220815135757-37a418bb8959
google.golang.org/protobuf v1.28.1
gopkg.in/yaml.v3 v3.0.0
```

## **Permission For Service Account**
### **For CAI creating Feed and fetching data of Assets**
```
Cloud Asset Owner (roles/cloudasset.owner) (on project level)
Cloud Asset Viewer (on organization level)
```
### **For SCC fetching data using api and creating Notification config**
```
Security Centre Admin Viewer (on organization level)
Security Center Notification Configurations Editor (roles/securitycenter.notificationConfigEditor) (on organization level)
```
### **For Creating Log Sink**
```
Logs Configuration Writer (roles/logging.configWriter) (on organization level)
Pub/Sub Admin (on Project level)
```



## **Ways To Use**
1. Docker Installation (Fully Automation).

## **Docker Installation**

### **Prerequisites**

1. For v3, users must have downloaded `config.yml`, `install` (binary file), templates (`filebeat.tmpl`, `logstash.tmpl` and `docker.tmpl`) and `dashboards.ndjson` files.

1. To keep the backward compatibility with the previous version (v1/v2), the user must have downloaded all required files of v3 along with `config.env` and `update`(binary file).

1. Users must have following Credntials:
    - If installing in non GCP-Cloud (AWS, Azure) Environment, [Workload Identity Federation](https://cloud.google.com/iam/docs/configuring-workload-identity-federation) Credentials JSON
    - If installing in GCP-Cloud Environment, No need to provide any Crednetials
    - If installing in Non-Cloud Environment, user need to provide Service-Account Credentials JSON.

1. [Elasticsearch](https://www.elastic.co/downloads/elasticsearch) and [Kibana](https://www.elastic.co/downloads/kibana) should be running.

1. [Docker](https://docs.docker.com/get-docker/) and [Docker Compose](https://docs.docker.com/compose/) should be installed and running.

### **Steps for upgrading from previous version to newer version**

Steps for upgrading from previous version (v1/v2) to new version (v3):

1. Run `./update` to convert `config.env` to `config.yml`.
1. Run Following Command if necessary.
    ```
    chmod +x update
    ./update
    ```
1. Check whether the `config.yml` file is created with configurations given in the `config.env` file or not.
1. If `config.yml` is not created, re run the `./update`
Once the `config.yml` file is created, users can add multiple organization configurations.

Follow the steps for Setup using Docker


#### **Provide Following Details**

1. Enter Configurations in `config.yml` file:
    1. Set Elastic-Host IP ( eg: 127.0.0.1 ) elasticsearch.host
    1. Set Elastic-Port  ( eg: 9200 ) elasticsearch.port
    1. Set Elastic-Username ( eg: elastic ) elasticsearch.username
    1. Set Elastic-Password  ( eg: changeme ) elasticsearch.password
    1. Set Elastic-CaCert  ( eg: path/to/cacert/elasticsearch.cer ) elasticsearch.cacert
    1. Set Proxy (Optional) ( http://user:password@proxyip:proxyport ) http_proxy  
    1. Set Kibana-Host IP ( eg: 127.0.0.1 ) kibana.host
    1. Set Kibana-Port  ( eg: 5601 ) kibana.port
    1. Set Kibana-Username ( eg: elastic ) kibana.username
    1. Set Kibana-Password  ( eg: changeme ) kibana.password
    1. Set Kibana-CaCert  ( eg: path/to/cacert/kibana.cer ) kibana.cacert
    1. Set As set Cron ( eg: “0 */45 * * * *” ) cron.asset
    1. Set Source Cron ( eg: “0 */45 * * * *” ) cron.source
        Reference: https://crontab.cronhub.io/
    1. Set Organization ID of your SCC ( eg: 12345678910 ) organizations.id
    1. Set client_secret.json file location in client_credential_path
    1. Set Project ID for Pub-Sub ( eg: project-id-12345 ) project.id
    1. Set Audit Log Pub-Sub Topic Name ( eg: AuditLog-topic) auditlog.topic_name
    1. Set Audit Log Pub-Sub Subscription Name ( eg: AuditLo-sub) auditlog.subscription_name
    1. Set Findings Pub-Sub Topic Name ( eg: Findings-topic) findings.topic_name
    1. Set Findings Start Date for migrating findings data (Optional) findings.start_date
      Format should be epoch (milliseconds) or an RFC3339 string. Few examples:
      2016-11-01T20:44:39Z
      2019-10-12T07:20:50.52Z
      2021-05-01T12:00:00+05:30
      1937-01-01T12:00:27.87+00:20
      https://www.rfc-editor.org/rfc/rfc3339#:~:text=2002%0A%0A%0A5.8.-,Examples,-Here%20are%20some
    1. Set Findings Pub-Sub Subscription Name ( eg: Findings-sub) findings.subscription_name
    1. Set Assets Resource Pub-Sub Topic Name ( eg: AssetResource-topic) asset.resource.topic_name
    1. Set Assets Resource Pub-Sub Subscription Name ( eg: AssetResource-sub) asset.resource.subscription_name
    1. Set Assets IamPolicy Pub-Sub Topic Name ( eg: AssetIamPolicy-topic) asset.iampolicy.topic_name
    1. Set Assets IamPolicy Pub-Sub Subscription Name ( eg: AssetIamPolicy-elk-sub) asset.iampolicy.subscription_name


Sample `config.yml` file for multiple organization configuration:
```
elasticsearch:
  host: <elasticsearch-host>
  port: <elasticsearch-port>
  username: <elasticsearch-username>
  password: <elasticsearch-password>
  cacert: <elasticsearch-cacert>
http_proxy: <http-proxy-link>
kibana:
  host: <kibana-host>
  port: <kibana-port>
  username: <kibana-username>
  password: <kibana-password>
  cacert: <kibana-cacert>
cron:
  asset: 0 */45 * * * * //Supports standard cron job
  source: 0 */45 * * * * //Supports standard cron job
organizations:
 - id: <organization-id>
   client_credential_path: <path-to-client_secret.json>
   project: 
    id: <project-id>
   auditlog:
    topic_name: <auditlog-topic-name>
    subscription_name: <auditlog-subscription-name>
   findings:
    topic_name: <findings-topic-name>
    start_date: <findings-start-date>
    subscription_name: <findings-subscription-name>
   asset:
    iampolicy:
     topic_name: asset_topic_name_iampolicy
     subscription_name: asset_subscription_name_iampolicy
    resource:
     topic_name: <asset-topic-name-resource>
     subscription_name: <asset-subscription-name-resource>
 - id: <organization-id>
   client_credential_path: <path-to-client_secret.json>
   project: 
    id: <project-id>
   auditlog:
    topic_name: <auditlog-topic-name>
    subscription_name: <auditlog-subscription-name>
   findings:
    topic_name: <findings-topic-name>
    start_date: <findings-start-date>
    subscription_name: <findings-subscription-name>
   asset:
    iampolicy:
     topic_name: <asset-topic-name-iampolicy>
     subscription_name: <asset-subscription-name-iampolicy>
    resource:
     topic_name: <asset-topic-name-resource>
     subscription_name: <asset-subscription-name-resource>
```

1. Place `config.yml`, `install` (binary file), templates (`filebeat.tmpl`, `logstash.tmpl` & `docker.tmpl`) & `dashboards.ndjson` files into your working directory i.e. install directory.
1. Run following command if necessary.
    ```
    chmod +x install
    ```
1. Run `install` file using command:  `./install`
1. Wait for the script to pull the docker image and setup container.
    
    For Example:
    ```
    ./install
    ```
1. You can check logs using following commands

    ```
    cat go.log (from your working directory)
    docker exec -it googlescc_elk ls (displays all Sink_<HashId> files)
    docker exec -it googlescc_elk cat Sink_<HashId> #to get sink service account
    ```

#### **Steps to add permission for Audit Logs:**

After above steps for installation are done,

Run the given command

`docker exec googlescc_elk cat Sink_<HashId>`

Note the serviceAccount Example: serviceAccount:<serviceAccount>

Add the permission for serviceAccount in topic <AuditLog PubSub Topic Name> for the role of `Pub/Sub Publisher`. 

Note: The Audit Logs won’t be ingested into elasticsearch if above permission is not given.


#### **Steps to see logs of GO Script**

Write Following commands:

```
cat go.log
docker exec -it googlescc_elk /bin/bash
cat Sink
cat Feed
```

#### **Steps for Uninstallation (Docker)**

1. Note the `container-id` of your docker container
    ```
    docker container ls
    ```
2. Stop the docker container using <container-id> and remove it
    ```
    docker stop googlescc_elk
    ```
3. Remove Docker Container
    ```
    docker rm [OPTIONS] googlescc_elk
    Use -f in options to remove it forcefully
    ```
4. Remove Docker Image
    ```
    docker rmi <docker-image>
    ```
5. Remove your working directory.

### **Optional**

Steps of cloning the dashboards in order to preserve the local changes after the upgrade:
```
Click on the CLONE option for cloning dashboard before upgrading ELK stack
```

### **Troubleshooting**
If any discrepancy is found in data collection then the user can check error logs for the app using the `go.log` file.

If the host instance does not have internet connectivity available then make sure the proxy is enabled and the proxy is reachable from the host instance.

If dashboards are not populating properly then please try increasing the time range in the kibana dashboards.

If new updates are not being fetched please check the Notification Config, Feed, and Sink are not deleted in GCP.

If the dashboards are deleted user can reupload the dashboards.ndjson file using the following steps:
```
Login to kibana move to Stack Management > Saved Objects > Import and Upload dashboards.ndjson file.
```
